"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/test/utest_compile.py $
$Id: utest_compile.py 26820 2005-05-19 19:49:46Z dbinger $
"""
from sancho.utest import UTest
from qpy.compile import translate_tokens, TemplateTransformer


class Test (UTest):

    def translation (self):
        src = ('def a [html] ():\n'
               '    "ok"\n')
        translated = translate_tokens(src)
        assert 'def a_h8(' in translated
        src = ('def a [plain] ():\n'
               '    "ok"\n')
        translated = translate_tokens(src)
        assert 'def a_u8(' in translated

    def transform(self):
        src = ('def a_h8 ():\n'
               '    "ok"\n')
        transformer = TemplateTransformer()
        result = transformer.parsesuite(src)

if __name__ == '__main__':
    Test()



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/test/utest_u8.py $
$Id: utest_u8.py 28062 2006-03-13 18:56:35Z dbinger $
"""
from sancho.utest import UTest, raises
from qpy import u8, stringify, html_escape_string

class Test (UTest):

    def test_u8(self):
        u = u8()
        assert str(u) == ''
        assert u == u8('')
        assert u == ''
        assert u == u8(u'')
        assert u == u8.from_list(['', u'', None])
        assert u8('<') == '<'
        u = u8('\xc3\xbc')
        assert u == u'\xfc'
        u = u8('ok')
        x = u.join(['a', u8('a')])
        assert isinstance(x, u8)
        assert x == u'aoka'
        assert 2 * u == 'okok'
        assert u * 2 == 'okok'

    def test_utils(self):
        raises(TypeError, stringify, 1)
        class A(object):
            def __unicode__(self):
                return 1
        raises(TypeError, stringify, A())
        A.__unicode__ = lambda x: u''
        raises(TypeError, stringify, A())
        class B(object):
            def __str__(self):
                return 1
        raises(TypeError, stringify, B())
        assert stringify(1) == '1'
        raises(TypeError, html_escape_string, 1)

    def test_quote_wrapper(self):
        assert u8("%0.2f") % 2 == '2.00'
        assert type(u8("%0.2f") % 2) is u8
        assert u8("%(a)s %(b)0.2f") % dict(a=u'a', b=2) == 'a 2.00'
        assert type(u8("%(a)s %(b)0.2f") % dict(a=u'a', b=2)) is u8
        assert u8("%(a)r %(b)0.2f") % dict(a=u'<', b=2) ==  "u'<' 2.00"
        assert type(u8("%(a)r %(b)0.2f")) is u8

    def test_pickle(self):
        from cPickle import loads, dumps
        s = u8('hello')
        pickle = '\x80\x02cqpy.c8\nu8\nq\x01X\x05\x00\x00\x00helloq\x02\x85\x81q\x03.'
        assert dumps(u8('hello'), 2) == pickle
        assert s == loads(dumps(u8(s), 2))



if __name__ == '__main__':
    Test()



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/compile.py $
$Id: compile.py 28532 2006-07-24 21:12:16Z dbinger $
"""
from compiler import misc, syntax
from compiler import pycodegen
from compiler.ast import AssName, Assign, Return, AugAssign, List, Getattr
from compiler.ast import Function, From, Module, Stmt, Name, CallFunc, Discard
from compiler.consts import OP_ASSIGN
from compiler.transformer import Transformer
from os import unlink, stat, listdir
from os.path import splitext, join
from token import ENDMARKER, NEWLINE, EQUAL
import re
import symbol
import sys

assert sys.hexversion >= 0x20300b1, 'requires Python 2.3 or newer'

source_suffix = '.qpy'

_template_re = re.compile(
    r"^(?P<indent>[ \t]*) def (?:[ \t]+)"
    r" (?P<name>[a-zA-Z_][a-zA-Z_0-9]*)"
    r" (?:[ \t]*) \[(?P<type>plain|html)\] (?:[ \t]*)"
    r" (?:[ \t]*[\(\\])",
    re.MULTILINE|re.VERBOSE)

def translate_tokens(buf):
    """
    def foo [plain] (...): -> def foo_u8(...):
    def foo [html] (...): -> def foo_h8(...):
    """
    def replacement(match):
        if match.group('type') == 'html':
            template_type = 'h8'
        else:
            template_type = 'u8'
        return '%sdef %s_%s(' % (match.group('indent'),
                                 match.group('name'),
                                 template_type)
    return  _template_re.sub(replacement, buf)


class TemplateTransformer(Transformer):

    def __init__(self, *args, **kwargs):
        Transformer.__init__(self, *args, **kwargs)
        # __template_type is a stack whose values are
        # "html", "plain", or None
        self.__template_type = []

    def _get_template_type(self):
        """Return the type of the function being compiled (
        "html", "plain", or None)
        """
        if self.__template_type:
            return self.__template_type[-1]
        else:
            return None

    def file_input(self, nodelist):
        doc = None # self.get_docstring(nodelist, symbol.file_input)
        if sys.hexversion >= 0x02050000:
            html_imp = From('qpy', [('u8', 'qpy_u8'), ('h8', 'qpy_h8')], 0)
        else:
            html_imp = From('qpy', [('u8', 'qpy_u8'), ('h8', 'qpy_h8')])
        stmts = [html_imp]
        for node in nodelist:
            if node[0] != ENDMARKER and node[0] != NEWLINE:
                self.com_append_stmt(stmts, node)
        return Module(doc, Stmt(stmts))

    def funcdef(self, nodelist):
        if len(nodelist) == 6:
            assert nodelist[0][0] == symbol.decorators
            decorators = self.decorators(nodelist[0][1:])
        else:
            assert len(nodelist) == 5
            decorators = None

        lineno = nodelist[-4][2]
        name = nodelist[-4][1]
        args = nodelist[-3][2]

        if name.endswith('_h8'):
            template_type = 'html'
            name = name[:-3]
        elif name.endswith('_u8'):
            template_type = 'plain'
            name = name[:-3]
        else:
            template_type = None
        self.__template_type.append(template_type)

        if template_type is None:
            n = Transformer.funcdef(self, nodelist)
        else:
            if args[0] == symbol.varargslist:
                names, defaults, flags = self.com_arglist(args[1:])
            else:
                names = defaults = ()
                flags = 0

            # qpy_accumulation = []
            assign_qpy_accumulation = Assign(
                [AssName('qpy_accumulation', OP_ASSIGN)], List([]))
            # qpy_append = qpy_accumulation.append
            assign_qpy_append = Assign(
                [AssName('qpy_append', OP_ASSIGN)],
                Getattr(Name('qpy_accumulation'), 'append'))

            # code for function
            code = self.com_node(nodelist[-1])

            if template_type == 'html':
                closer = 'qpy_h8'
            else:
                closer = 'qpy_u8'
            ret = Return(CallFunc(Getattr(Name(closer), 'from_list'),
                                  [Name('qpy_accumulation')]))

            # wrap original function code
            new_code = Stmt([
                assign_qpy_accumulation,
                assign_qpy_append,
                code,
                ret
                ])
            doc = None
            if sys.hexversion >= 0x20400a2:
                n = Function(decorators, name, names, defaults, flags, doc,
                             new_code)
            else:
                n = Function(name, names, defaults, flags, doc, new_code)
            n.lineno = lineno

        self.__template_type.pop()
        return n

    def expr_stmt(self, nodelist):
        if self._get_template_type() is None:
            return Transformer.expr_stmt(self, nodelist)
        # Instead of discarding objects on the stack, call
        # qpy_append(obj)
        exprNode = self.com_node(nodelist[-1])
        if len(nodelist) == 1:
            n = CallFunc(Name('qpy_append'), [exprNode])
            if hasattr(exprNode, 'lineno'):
                n.lineno = exprNode.lineno
            n = Discard(n)
        elif nodelist[1][0] == EQUAL:
            nodes = [ ]
            for i in range(0, len(nodelist) - 2, 2):
                nodes.append(self.com_assign(nodelist[i], OP_ASSIGN))
            n = Assign(nodes, exprNode)
            n.lineno = nodelist[1][2]
        else:
            lval = self.com_augassign(nodelist[0])
            op = self.com_augassign_op(nodelist[1])
            n = AugAssign(lval, op[1], exprNode)
            n.lineno = op[2]
        return n

    def atom_string(self, nodelist):
        const_node = Transformer.atom_string(self, nodelist)
        template_type = self._get_template_type()
        if template_type == "html":
            return CallFunc(Name('qpy_h8'), [const_node])
        elif template_type == "plain":
            return CallFunc(Name('qpy_u8'), [const_node])
        else:
            return const_node

def parse(buf, filename):
    try:
        return TemplateTransformer().parsesuite(translate_tokens(buf))
    except SyntaxError, e:
        raise SyntaxError(str(e), (filename, e.lineno, e.offset, e.text))


class Template(pycodegen.Module):

    def _get_tree(self):
        tree = parse(self.source, self.filename)
        misc.set_filename(self.filename, tree)
        syntax.check(tree)
        return tree

def compile(source_name):
    base, ext = splitext(source_name)

    source_file = open(source_name)
    source = source_file.read()
    source_file.close()
    output_name = base + '.pyc'
    output = open(output_name, 'wb')
    try:
        template = Template(source, source_name)
        template.compile()
        template.dump(output)
    except:
        output.close()
        unlink(output_name)
        raise
    else:
        output.close()
        return template.code

def timestamp(filename):
    try:
        s = stat(filename)
    except OSError:
        return None
    return s.st_mtime

def compile_qpy_file(source):
    """(source:str)
    Compile the given filename if it is a .qpy file and if it does not already
    have an up-to-date .pyc file.
    """
    if source[-4:] == '.qpy':
        compile_time = timestamp(source[:-4] + '.pyc')
        if compile_time is not None:
            source_time = timestamp(source)
            if compile_time >= source_time:
                return # already up-to-date
        compile(source)

def compile_qpy_files(path):
    """(path:str)
    Compile the .qpy files in the given directory.
    """
    for name in listdir(path):
        if name[-4:] == '.qpy':
            compile_qpy_file(join(path, name))

def main():
    args = sys.argv[1:]
    if not args:
        print "no files to compile"
    else:
        for filename in args:
            compile(filename)

if __name__ == "__main__":
    main()



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/c8.py $
$Id: c8.py 28069 2006-03-14 14:45:02Z dbinger $
"""

def html_escape_string(s):
    """(basestring) -> basestring
    Replace characters '&', '<', '>', '"' with HTML entities.
    The type of the result is either str or unicode.
    """
    if not isinstance(s, basestring):
        raise TypeError, 'string object required'
    s = s.replace("&", "&amp;")
    s = s.replace("<", "&lt;")
    s = s.replace(">", "&gt;")
    s = s.replace('"', "&quot;")
    return s

def stringify(obj):
    """(obj) -> basestring
    Return a string version of `obj`.  This is like str(), except
    that it tries to prevent turning str instances into unicode instances.
    The type of the result is either str or unicode.
    """
    tp = type(obj)
    if issubclass(tp, basestring):
        return obj
    elif hasattr(tp, '__unicode__'):
        s = tp.__unicode__(obj)
        if not isinstance(s, basestring):
            raise TypeError, '__unicode__ did not return a string'
        return s
    elif hasattr(tp, '__str__'):
        s = tp.__str__(obj)
        if not isinstance(s, basestring):
            raise TypeError, '__str__ did not return a string'
        return s
    else:
        return str(obj)


class _quote_wrapper (object):
    """
    Not for outside code.
    For values that are used as arguments to the % operator, this allows
    str(value) and repr(value), if called as part of the formatting, to
    produce quoted results.
    """
    __slots__ = ['value', 'quote']

    def __init__(self, value, quote):
        self.value = value
        self.quote = quote

    def __str__(self):
        return self.quote(self.value)

    def __repr__(self):
        return self.quote(`self.value`)

    def __getitem__(self, key):
        return _quote_wrap(self.value[key], self.quote)

def _quote_wrap(x, quote):
    """(x, quote:h8_quote|u8_quote) -> _quote_wrapper | x
    Not for outside code.
    Return a value v for which str(v) and repr(v) will produce quoted strings,
    or, if x is a dict, for which str(v[key]) and repr(v[key]) will produce
    quoted strings.
    """
    if isinstance(x, (int, long, float)):
        return x
    else:
        return _quote_wrapper(x, quote)


class u8 (unicode):
    """
    This is a subclass of unicode for which the constructor defaults to utf-8
    encoding and returns the empty instance when the argument is None or not given.

    u8.from_list(x) is the same as u8('').join(x).
    """
    __slots__ = []

    def __new__(klass, string='', encoding='utf-8', errors="strict"):
        if (string is None or
            (isinstance(string, basestring) and len(string) == 0)):
            if klass is u8:
                return _u8_empty
            if klass is h8:
                return _h8_empty
        if isinstance(string, unicode):
            return unicode.__new__(klass, string)
        return unicode.__new__(klass, string, encoding, errors)

    def __str__(self):
        """() -> str
        Returns the utf-8 encoding.
        """
        return self.encode('utf-8')

    def __add__(self, other):
        """(other) -> instance of this class
        Like unicode.__add__, except that the argument is quoted and
        the result is an instance of this class.
        """
        return self.__class__(unicode.__add__(self, self.quote(other)))

    def __radd__(self, other):
        """(other) -> instance of this class
        Like unicode.__radd__, except that the argument is quoted and
        the result is an instance of this class.
        """
        return self.__class__(unicode.__add__(self.quote(other), self))

    def __mul__(self, other):
        """(other) -> instance of this class
        Like unicode.__mul__, except that the result is an instance of this class.
        """
        return self.__class__(unicode.__mul__(self, other))

    def __rmul__(self, other):
        """(other) -> instance of this class
        Like unicode.__rmul__, except that the result is an instance of this class.
        """
        return self.__class__(unicode.__mul__(self, other))

    def __mod__(self, other):
        """(other) -> instance of this class
        Like unicode.__mod__, except that the inserted values are quoted.
        """
        if isinstance(other, tuple):
            target = tuple(_quote_wrap(item, self.quote) for item in other)
        else:
            target = _quote_wrap(other, self.__class__.quote)
        return self.__class__(unicode.__mod__(self, target))

    def __rmod__(self, other):
        """(other) -> instance of this class
        Like unicode.__rmod__, except that the other value is quoted.
        """
        return self.quote(other) % self

    def join(self, items):
        """(items:list) -> instance of this class
        Like unicode join, except that the items in the list are quoted first.
        """
        quote = self.__class__.quote
        return self.__class__(unicode.join(self, (quote(item) for item in items)))

    @classmethod
    def quote(klass, s):
        """(s:anything) -> instance of this class
        Return an instance of this class, based on s.
        """
        if isinstance(s, klass):
            return s
        if s is None:
            return klass()
        return klass(stringify(s))

_u8_empty = unicode.__new__(u8, '')
u8.from_list = _u8_empty.join


class h8 (u8):
    """
    This subclass of u8 is designated as needing no more html quoting.
    Some operations (the ones with __<op>__ overrides in the u8 class) that
    combine this with non-h8 instances quote the non-h8 instances and produce
    h8 instances.

    h8.from_list(x) is the same as h8('').join(x).
    """
    __slots__ = []

    @classmethod
    def quote(klass, s):
        """(s:anything) -> h8
        Return an h8 instance, based on s.  Escape HTML characters as needed.
        """
        if isinstance(s, klass):
            return s
        if s is None:
            return klass()
        return klass(html_escape_string(stringify(s)))

_h8_empty = unicode.__new__(h8, '')
h8.from_list = _h8_empty.join

__all__ = ['u8', 'h8', 'html_escape_string', 'stringify']




"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/__init__.py $
$Id: __init__.py 28581 2006-08-14 22:11:40Z dbinger $
"""
__version__ = "1.3"

# external code should import these from here.
from qpy.c8 import u8, h8, stringify, html_escape_string
[u8, h8, stringify, html_escape_string] # for qpcheck.py.





#!/usr/bin/env python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/qpcheck.py $
$Id: qpcheck.py 28533 2006-07-24 21:12:39Z dbinger $
"""
from compiler.ast import Function, Class, AssName, Name
from compiler.ast import Import, From, ListComp, Lambda, GenExprInner
from optparse import OptionParser
from os import listdir
from os.path import join, isdir, isfile, realpath, basename
from qpy.compile import parse
from types import ModuleType
import sys

def get_source_file(module):
    """(module:str|ModuleType) -> str
    Identify source file for module.
    """
    if type(module) is str:
        try:
            module_file = __import__(module,{},{},['__file__']).__file__
        except (ImportError, ValueError):
            module_file = module
    else:
        assert type(module) is ModuleType
        module_file = module.__file__
    if module_file[-1] in 'oc':
        module_file = module_file[:-1]
    return realpath(module_file)

def file_tree(module, out):
    if module is None:
        source_file = '<stdin>'
        raw_source = sys.stdin.read()
    else:
        try:
            source_file = get_source_file(module)
            raw_source = open(source_file, 'rU').read()
        except Exception, e:
            raise e
            source_file = '<str>'
            raw_source = module
    try:
        root_node = parse(raw_source, source_file)
    except:
        out.write(source_file + '\n')
        raise
    return source_file, root_node


def check(module, verbose=False, check_imports=True, out=sys.stdout,
          approved_stars=('types')):
    if verbose:
        print "CHECK", module
    def arg_names (node):
        result = []
        def flatten(item):
            for x in item:
                if type(x) is tuple:
                    flatten(x)
                else:
                    result.append(x)
        flatten(node.argnames)
        return result

    unresolved = []
    namespace = set()

    def traverse(node, namestack):
        childNodes = node.getChildNodes()
        if isinstance(node, Lambda):
            for child in node.defaults:
                traverse(child, namestack)
            prevnode, prevprefix, prevspace = namestack[-1]
            namestack = namestack + [[node, prevprefix + ['<lambda>'],
                                      set(arg_names(node))]]
            childNodes = [node.code]
        if isinstance(node, Function):
            for child in node.defaults:
                traverse(child, namestack)
            childNodes = [node.code]
            prevnode, prevprefix, prevspace = namestack[-1]
            namestack = namestack + [[node, prevprefix + [node.name],
                                      set([node.name] + arg_names(node))]]
            if isinstance(prevnode, Class):
                namestack[-1][2].add(prevnode.name)
        if isinstance(node, Class):
            prefix = namestack[-1][1]
            namestack = namestack + [[node, prefix + [node.name], set([])]]
        if isinstance(node, Name):
            name = node.name
            if (name not in namestack[-1][2] and # local
                name not in namestack[0][2]): # global
                # search nested
                for snode, sprefix, space in namestack[1:-1]:
                    if isinstance(snode, Function) and name in space:
                        break
                else:
                    prefix = namestack[-1][1]
                    unresolved.append((node.lineno, prefix, name))
        if isinstance(node, AssName):
            namestack[-1][2].add(node.name)
            if len(namestack) == 1:
                namespace.add(node.name)
        if isinstance(node, Import):
            for name, as_name in node.names:
                namestack[-1][2].add(as_name or name)
                name_split = name.split('.')
                for index in range(len(name_split)):
                    namespace.add('.'.join(name_split[:index+1]))
        if isinstance(node, From):
            for name, as_name in node.names:
                if name == '*':
                    if verbose:
                        prefix = namestack[-1][1]
                        unresolved.append((node.lineno, prefix, name))
                    if node.modname in approved_stars:
                        module = __import__(node.modname)
                        names = namestack[-1][2]
                        for name in dir(module):
                            if not name.startswith('_'):
                                names.add(name)
                else:
                    namestack[-1][2].add(as_name or name)
        if isinstance(node, (ListComp, GenExprInner)):
            childNodes = tuple(node.quals +  [node.expr])
        for child in childNodes:
            traverse(child, namestack)
        if isinstance(node, (Function, Class)):
            prevnode, prefix, preset = namestack[-2]
            preset.add(node.name)
            namespace.add('.'.join(prefix + [node.name]))
            if isinstance(node, Class):
                prevnode, prefix, preset = namestack[-1]
                for name in preset:
                    namespace.add('.'.join(prefix + [name]))

    filename, root_node = file_tree(module, out)
    import __main__
    traverse(root_node,
             [[root_node, [], set(dir(__main__) +
                                  dir(__main__.__builtins__))]])
    for lineno, prefix, name in unresolved:
        if name=='__path__' and filename.endswith('__init__.py'):
            continue
        if name in namespace:
            if not verbose:
                continue # skip these
            message = "used before definition"
        else:
            message = "not found"
        out.write("%s:%s: %s %s.\n" % (
            filename, lineno, '.'.join(prefix + [name]), message))

    if not check_imports:
        return

    unused_imports={}
    def walk(ast):
        if isinstance(ast, Import):
            for name, as_name in ast.names:
                if '.' in name:
                    # import a.b
                    # Usage of these names is not currently detected here,
                    # so we won't complain about it.
                    pass
                else:
                    unused_imports[as_name or name] = name
        if isinstance(ast, From):
            for name, as_name in ast.names:
                if '*' != name or verbose:
                    unused_imports[as_name or name] = name
        if isinstance(ast, Name):
            if ast.name in unused_imports:
                del unused_imports[ast.name]
        for child in ast.getChildNodes():
            walk(child)
    walk(root_node)
    for as_name, name in unused_imports.items():
        if as_name == name:
            if name not in ('SitePublisher', 'SiteRootDirectory'):
                out.write("%s: %s\n" % (filename, name))
        if as_name != name:
            if (not as_name.startswith('_') and
                not as_name in ('qpy_h8', 'qpy_u8')):
                # If you want to import something for a side-effect,
                # but you don't want this warning, make the import
                # an 'as' import and make the name start with underscore.
                out.write("%s: %s as %s\n" % (filename, name, as_name))


if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('--verbose', action='store_true', default=False)
    parser.set_description(
        'Check python source files for unknown name errors '
        'and for unused imports. '
        'The arguments name files to check or '
        'directories to check recursively. '
        'If no arguments are given, the current directory '
        'is checked. '
        'If there is a recursion, it excludes ".svn", "build", '
        'and "dist" directories, and includes files that end '
        'in ".py", ".qpy", or ".ptl".')
    (options, args) = parser.parse_args()
    if args:
        todo = args
    else:
        todo = ['.']
    while todo:
        arg = todo.pop()
        if basename(arg) in ['.svn', 'dist', 'build']:
            continue
        elif (isfile(arg) and
              arg in sys.argv or
              (arg.endswith('.py') or
               arg.endswith('.qpy') or
               arg.endswith('.ptl'))):
            check(arg, verbose=options.verbose)
        elif isdir(arg):
            todo.extend([join(arg, item) for item in listdir(arg)])



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/setup.py $
$Id: setup.py 28588 2006-08-15 17:11:08Z dbinger $
"""
from distutils.core import setup
from distutils.command.build_py import build_py
from distutils.extension import Extension
from glob import glob
import os, sys


class qpy_build_py(build_py):

    def find_package_modules(self, package, package_dir):
        self.check_package(package, package_dir)
        module_files = (glob(os.path.join(package_dir, "*.py")) +
                        glob(os.path.join(package_dir, "*.qpy")))
        modules = []
        setup_script = os.path.abspath(self.distribution.script_name)
        for f in module_files:
            abs_f = os.path.abspath(f)
            module = os.path.splitext(os.path.basename(f))[0]
            modules.append((package, module, f))
        return modules

    def build_module(self, module, module_file, package):
        if type(package) is str:
            package = package.split('.')
        elif type(package) not in (list, tuple):
            raise TypeError, \
                  "'package' must be a string (dot-separated), list, or tuple"
        # Now put the module source file into the "build" area.
        outfile = self.get_module_outfile(self.build_lib, package, module)
        if module_file.endswith(".qpy"): 
            outfile = outfile[0:outfile.rfind('.')] + ".qpy"
        dir = os.path.dirname(outfile)
        self.mkpath(dir)
        return self.copy_file(module_file, outfile, preserve_mode=0)


if __name__ == '__main__':
    from __init__ import __version__
    if 'sdist' in sys.argv:
        import re

        # Make sure that version numbers have all been updated.
        PAT = re.compile(r'\b%s\b' % re.escape(__version__))
        assert len(PAT.findall(open("LICENSE.txt").read())) == 14, __version__
        assert PAT.search(open("CHANGES.txt").readline()), __version__
        assert len(PAT.findall(open("README.txt").read())) == 0, __version__

        # Make sure that copyright statements are current.
        from datetime import datetime
        year = datetime.now().year
        copyright = "Copyright (c) Corporation for National Research Initiatives %s" % year
        assert open("README.txt").read().count(copyright) == 1
    setup(
        name="qpy",
        version=__version__,
        description="unicode/html content in functions",
        author="CNRI",
        author_email="webmaster@mems-exchange.org",
        url="http://www.mems-exchange.org/software/qpy/",
        license="see LICENSE.txt",
        package_dir=dict(qpy=os.curdir),
        packages=['qpy'],
        scripts=['qpcheck.py', 'qpyrun.py'],
        ext_modules=[Extension(name="qpy.c8", sources=["c8.c"])],
        cmdclass= {'build_py': qpy_build_py},
        long_description=(
        "Qpy provides a convenient mechanism for generating safely-quoted html "
        "text from python code.  It does this by implementing a quoted-string "
        "data type and a modification of the python compiler.  (This main idea "
        "comes from Quixote's htmltext/PTL.)")
        )



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qpy/test/utest_h8.py $
$Id: utest_h8.py 27679 2005-11-08 16:42:04Z dbinger $
"""
from sancho.utest import UTest
from qpy import u8, h8

class Test (UTest):

    def test_h8(self):
        u = h8()
        assert str(u) == ''
        assert u == h8('')
        assert u == u8('')
        assert u == ''
        assert h8('<') == '<'
        assert h8.quote('<') == '&lt;'
        u += 1
        u += None
        assert u == '1'
        assert h8('a%s') % '&' == 'a&amp;'
        assert h8('a%(a)s%(b)s') % dict(a='&', b=h8('<')) == 'a&amp;<'
        assert h8('a%s%s') % ('&', h8('<')) == 'a&amp;<'

    def test_a(self):
        s = h8('<') + '<'
        assert isinstance(s, h8) and s == '<&lt;'
        s = '<' + h8('<')
        assert isinstance(s, h8) and s == '&lt;<', '__radd__ not working.'
        s = h8('%s') % '<'
        assert isinstance(s, h8) and  s == '&lt;'
        s = h8('%s') % h8('<')
        assert isinstance(s, h8) and s == '<'
        s = h8('%(a)s') % dict(a='<')
        assert isinstance(s, h8) and s == '&lt;'
        # mod into str does not produce an h8.
        s = '%s' % h8('a')
        assert isinstance(s, unicode) and not isinstance(s, h8) and s == 'a'
        s = u'%s' % h8('a')
        assert isinstance(s, unicode) and isinstance(s, h8) and s == 'a'
        s = h8('a')
        s += '<'
        assert s == 'a&lt;'

    def test_quote_wrapper(self):
        assert h8("%0.2f") % 2 == '2.00'
        assert type(h8("%0.2f") % 2) is h8
        assert h8("%(a)s %(b)0.2f") % dict(a=u'<', b=2) == '&lt; 2.00'
        assert type(h8("%(a)s %(b)0.2f") % dict(a=u'a', b=2)) is h8
        assert h8("%(a)r %(b)0.2f") % dict(a=u'<', b=2) ==  "u'&lt;' 2.00"
        assert type(h8("%(a)r %(b)0.2f")) is h8

    def test_pickle(self):
        from cPickle import loads, dumps
        s = h8('hello')
        pickle = '\x80\x02cqpy.c8\nh8\nq\x01X\x05\x00\x00\x00helloq\x02\x85\x81q\x03.'
        assert dumps(h8('hello'), 2) == pickle
        assert s == loads(dumps(h8(s), 2))

if __name__ == '__main__':
    Test()
